import { app, shell, BrowserWindow, ipcMain } from 'electron'
import { join } from 'path'
import { electronApp, optimizer, is } from '@electron-toolkit/utils'
import icon from '../../resources/icon.png?asset'

// 模拟配置的modbus配置单
const modbusConfig = {
  name: [
    '风速',
    '风向',
    '温度',
    '湿度',
    '气压',
    '光照',
    '雨量',
    'PM2.5',
    'PM10',
    '噪音',
  ], // 名称
}
var ModbusRTU = require('modbus-serial')
var client = new ModbusRTU()
// 寄存器循环
var time = null
// 主窗口
var mainWindow
// 登录窗口
var loginWindow
// 连接后读取modbus从机的连接
var readClient

// 读取寄存器
ipcMain.on('read', (event, message) => {
  read(event, message)
  console.log('pong')
})

// 登录后开启modbus连接
ipcMain.on('login', (event, message) => {
  // login(event, message)
  mainWindow.webContents.send('loginReply', 1)
  loginWindow.close()
})

// 登录
// function login(event, message) {
//   // 登录业务代码
//   console.log('login', message, event)
// }

// 接收渲染进程发送的link请求
ipcMain.on('link', (event, message) => {
  link(event, message)
})

// 连接modbus
function link(event, message) {
  // 连接modbus
  client.connectTCP(message.ip, { port: 502 })
  // 连接成功后读取寄存器方法
  readClient = (event, message) => {
    client.readHoldingRegisters(message.startAddress, message.quantity, function (err, data) {
      if (err) {
        event.reply('readReply', err)
        return
      }
      event.reply('readReply', dataHandle(data.data))
    })
  }
  read(event, message)
}

// 通过功能码03读取寄存器
function read(event, message) {
  if (!(time == null)) {
    console.log('重复读取数据', time)
    return
  }
  // 循环读取寄存器
  time = setInterval(() => {
    readClient(event, message)
  }, 500)
}

// 数据处理
function dataHandle(data) {
  let result = []
  for (let i = 0; i < modbusConfig.name.length; i++) {
    result[i] = {
      name: modbusConfig.name[i],
      data: data[i]
    }
  }

  return result
}

// 停止modbus读取
ipcMain.on('stop', (event) => {
  clearInterval(time)
  time = null
  console.log('stop')
  // 返回数据给渲染进程
  event.reply('readReply', [])
})

// 创建主窗口
function createWindow() {
  // Create the browser window.
  mainWindow = new BrowserWindow({
    width: 900,
    height: 670,
    show: false,
    autoHideMenuBar: true,
    ...(process.platform === 'linux' ? { icon } : {}),
    webPreferences: {
      preload: join(__dirname, '../preload/index.js'),
      sandbox: false
    }
  })
  mainWindow.on('ready-to-show', () => {
    mainWindow.show()
  })

  mainWindow.webContents.setWindowOpenHandler((details) => {
    shell.openExternal(details.url)
    return { action: 'deny' }
  })

  // HMR for renderer base on electron-vite cli.
  // Load the remote URL for development or the local html file for production.
  if (is.dev && process.env['ELECTRON_RENDERER_URL']) {
    mainWindow.loadURL(process.env['ELECTRON_RENDERER_URL'])
  } else {
    mainWindow.loadFile(join(__dirname, '../renderer/index.html'))
  }
}

// 创建子窗口
function createChildWindow() {
  loginWindow = new BrowserWindow({
    parent: mainWindow,
    width: 500,
    height: 400,
    show: false,
    autoHideMenuBar: true,
    ...(process.platform === 'linux' ? { icon } : {}),
    webPreferences: {
      preload: join(__dirname, '../preload/index.js'),
      sandbox: false
    }
  })

  loginWindow.on('ready-to-show', () => {
    loginWindow.show()
  })

  loginWindow.webContents.setWindowOpenHandler((details) => {
    // console.log('details.url', details.url)
    shell.openExternal(details.url)
    return { action: 'deny' }
  })

  // HMR for renderer base on electron-vite cli.
  // Load the remote URL for development or the local html file for production.
  if (is.dev && process.env['ELECTRON_RENDERER_URL']) {
    // console.log(process.env['ELECTRON_RENDERER_URL'] + '/#/login')
    loginWindow.loadURL(process.env['ELECTRON_RENDERER_URL'] + '/#/login')
  } else {
    // loginWindow.loadURL(join(__dirname, '../renderer/login.html'))
  }
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.whenReady().then(() => {
  // Set app user model id for windows
  electronApp.setAppUserModelId('com.electron')

  // Default open or close DevTools by F12 in development
  // and ignore CommandOrControl + R in production.
  // see https://github.com/alex8088/electron-toolkit/tree/master/packages/utils
  app.on('browser-window-created', (_, window) => {
    optimizer.watchWindowShortcuts(window)
  })

  createWindow()
  createChildWindow()
  app.on('activate', function () {
    // On macOS it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (BrowserWindow.getAllWindows().length === 0) createWindow()
  })
})

// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit()
  }
})

// In this file you can include the rest of your app"s specific main process
// code. You can also put them in separate files and require them here.
